﻿using System;
using System.Collections.Generic;
using System.IO;

using TextMateSharp.Grammars;
using TextMateSharp.Internal.Themes.Reader;
using TextMateSharp.Registry;
using TextMateSharp.Themes;

namespace TextMateSharp
{
    class Program
    {
        static void Main(string[] args)
        {
            string fileToParse = Path.GetFullPath(args[0]);

            var sourceTextLines = File.ReadAllText(fileToParse).Replace("\n", "\r\n").Split(Environment.NewLine);

            var textLinesStartWithSecondLine = sourceTextLines[1..];

            Console.WriteLine("Start in first line");

            StartAnalyzeText(args, sourceTextLines);

            Console.WriteLine("Start with second line");

            StartAnalyzeText(args, textLinesStartWithSecondLine);
        }

        private static void StartAnalyzeText(string[] args, string[] textLines)
        {
            string grammarFile = Path.GetFullPath(args[1]);
            string themeFile = Path.GetFullPath(args[2]);

            IRegistryOptions options = new DemoRegistryOptions(grammarFile, themeFile);

            Registry.Registry registry = new Registry.Registry(options);

            IGrammar grammar = registry.LoadGrammar("source.json");

            StackElement ruleStack = null;

            foreach (string line in textLines)
            {
                ITokenizeLineResult result = grammar.TokenizeLine(line, ruleStack);

                ruleStack = result.RuleStack;

                foreach (IToken token in result.Tokens)
                {
                    int startIndex = (token.StartIndex > line.Length) ?
                        line.Length : token.StartIndex;
                    int endIndex = (token.EndIndex > line.Length) ?
                        line.Length : token.EndIndex;

                    var keyName = line.Substring(startIndex, endIndex - startIndex);

                    if (keyName == "name")
                    {
                        Console.WriteLine(string.Format(
                        "  - token from {0} to {1} -->{2}<-- with scopes {3}",
                        startIndex,
                        endIndex,
                        keyName,
                        string.Join(",", token.Scopes)));
                    }
                }
            }
        }

        class DemoRegistryOptions : IRegistryOptions
        {
            private string _grammarFile;
            private string _themeFile;

            internal DemoRegistryOptions(string grammarFile, string themeFile)
            {
                _grammarFile = grammarFile;
                _themeFile = themeFile;
            }

            public string GetFilePath(string scopeName)
            {
                if (scopeName == "./dark_vs.json")
                    return Path.Combine(Path.GetDirectoryName(_themeFile), "dark_vs.json");
                if (scopeName == "source.json")
                    return Path.Combine(Path.GetDirectoryName(_grammarFile), "JSON.tmLanguage.json");

                return null;
            }

            public ICollection<string> GetInjections(string scopeName)
            {
                return null;
            }

            public Stream GetInputStream(string scopeName)
            {
                return new FileStream(
                    GetFilePath(scopeName),
                    FileMode.Open,
                    FileAccess.Read,
                    FileShare.ReadWrite);
            }

            public IRawTheme GetTheme()
            {
                int ini = Environment.TickCount;

                using (StreamReader reader = new StreamReader(_themeFile))
                {
                    IRawTheme result = ThemeReader.ReadThemeSync(reader);
                    Console.WriteLine("Loaded {0} in {1}ms.",
                        Path.GetFileName(_themeFile),
                        Environment.TickCount - ini);
                    return result;
                }
            }
        }
    }
}
